/* inverse discrete cosine transform on 8x8 block
 * Copyright (C) 2001,2002,2003  David A. Schleef <ds@schleef.org>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA 02111-1307, USA.
 */


#include <specialib/sl_types.h>
#include <math.h>


#define BLOCK8x8_F64(ptr, stride, row, column) \
	(*((double *)((void *)ptr + stride*row) + column))

#define BLOCK8x8_PTR_F64(ptr, stride, row, column) \
	((double *)((void *)ptr + stride*row) + column)

#define BLOCK8x8_S16(ptr, stride, row, column) \
	(*((int16_t *)((void *)ptr + stride*row) + column))

/**
 * sl_idct8x8_F64:
 * @dst: destination array
 * @dstr: destination row stride
 * @src: source array
 * @sstr: source row stride
 * #arch C
 *
 * Perform an inverse discrete cosine transform on an 8x8 block.
 */
SL_FUNC void _sl_idct8x8_F64__c_slow(double *dest, int dstr, double *src, int sstr)
{
	static double idct_coeff[8][8];
	static int idct_coeff_init = 0;
	int i,j,k,l;
	double tmp1,tmp2;

	if(!idct_coeff_init){
		double scale;

		for(i=0;i<8;i++){
			scale = (i==0) ? sqrt(0.125) : 0.5;
			for(j=0;j<8;j++){
				idct_coeff[j][i] = scale *
					cos((M_PI/8)*i*(j+0.5));
			}
		}
		idct_coeff_init = 1;
	}

	for(i=0;i<8;i++){
		for(j=0;j<8;j++){
			tmp1 = 0;
			for(k=0;k<8;k++){
				tmp2 = 0;
				for(l=0;l<8;l++){
					tmp2 += idct_coeff[j][l] *
						BLOCK8x8_F64(src,sstr,k,l);
				}
				tmp1 += idct_coeff[i][k] * tmp2;
			}
			BLOCK8x8_F64(dest,dstr,i,j) = tmp1;
		}
	}
}

SL_FUNC void _sl_idct8x8_F64__c(double *dest, int dstr, double *src, int sstr)
{
	int i;
	double tmp[64];
	int tmpstr = 8*sizeof(double);

	for(i=0;i<8;i++){
		/* FIXME: should use optimal function, not __c */
		_sl_idct8_F64__c(
			BLOCK8x8_PTR_F64(tmp,tmpstr,i,0), sizeof(double),
			BLOCK8x8_PTR_F64(src,sstr,i,0), sizeof(double));
	}
	for(i=0;i<8;i++){
		_sl_idct8_F64__c(
			BLOCK8x8_PTR_F64(dest,dstr,0,i), dstr,
			BLOCK8x8_PTR_F64(tmp,tmpstr,0,i), tmpstr);
	}
}




/**
 * sl_idct8x8_S16:
 * @dst: destination array
 * @dstr: destination row stride
 * @src: source array
 * @sstr: source row stride
 * #arch C
 *
 * Perform an inverse discrete cosine transform on an 8x8 block.
 */
SL_FUNC void _sl_idct8x8_S16__c_slow(int16_t *dest, int dstr, int16_t *src,
	int sstr)
{
	double s[64], d[64];

	/* FIXME: should use optimal function, not __c */
	_sl_convert8x8_F64_S16__c(s,8*sizeof(double),src,sstr);
	_sl_idct8x8_F64__c(d,8*sizeof(double),s,8*sizeof(double));
	_sl_convert8x8_S16_F64__c(dest,dstr,d,8*sizeof(double));
}

